from lokit import Office
import argparse
import os

LO_PATH = "/opt/libreoffice4.3/program"

if __name__ == '__main__':
    parser = argparse.ArgumentParser(prog='lokitconv.py',
            description='Requires LibreOffice 4.3.0')
    parser.add_argument('-f', '--format',
            help='Known formats include:\
                For text documents: doc docx fodt html odt ott pdf txt xhtml')
    parser.add_argument('-o', '--options',
            help='Filter options, known options include: SkipImages')
    parser.add_argument('input_file')
    parser.add_argument('output_file')

    args = parser.parse_args()
    lo_path = os.getenv('LO_PATH', LO_PATH)
    lo = Office(lo_path)
    doc = lo.documentLoad(args.input_file)
    doc.saveAs(args.output_file, fmt=args.format, options=args.options)

    os._exit(0)



from setuptools import setup, find_packages
import os

VERSION = "0.8.1"

CLASSIFIERS = [
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: System Administrators',
    'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Python Modules',
    'Topic :: Office/Business :: Office Suites',
]

setup(
    author="Riccardo Magliocchetti",
    author_email="riccardo.magliocchetti@gmail.com",
    name='pylokit',
    version=VERSION,
    description='Python CFFI wrapper for LibreOfficeKit',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.rst')).read(),
    url="https://github.com/xrmx/pylokit",
    license='MPL 2.0',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'cffi',
        'six',
    ],
    test_suite='pylokit.tests',
    packages=find_packages(),
    include_package_data=True,
    zip_safe = False,
)



# Python cffi wrapper for LibreOfficeKit
#
# Copyright (C) 2014 Riccardo Magliocchetti
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

from cffi import FFI
import six
import os


TARGET_LIB = ("libsofficeapp.so", "libmergedlo.so")

LOKIT_CDEFS = """
typedef struct _LibreOfficeKit LibreOfficeKit;
typedef struct _LibreOfficeKitClass LibreOfficeKitClass;

typedef struct _LibreOfficeKitDocument LibreOfficeKitDocument;
typedef struct _LibreOfficeKitDocumentClass LibreOfficeKitDocumentClass;

struct _LibreOfficeKit
{
    LibreOfficeKitClass* pClass;
};

struct _LibreOfficeKitClass
{
  size_t  nSize;

  void                    (*destroy)       (LibreOfficeKit *pThis);
  LibreOfficeKitDocument* (*documentLoad)  (LibreOfficeKit *pThis, const char *pURL);
  char*                   (*getError)      (LibreOfficeKit *pThis);
};

struct _LibreOfficeKitDocument
{
    LibreOfficeKitDocumentClass* pClass;
};

struct _LibreOfficeKitDocumentClass
{
  size_t  nSize;

  void (*destroy)   (LibreOfficeKitDocument* pThis);
  int (*saveAs)     (LibreOfficeKitDocument* pThis,
                     const char *pUrl,
                     const char *pFormat,
                     const char *pFilterOptions);
};
LibreOfficeKit *libreofficekit_hook(const char* install_path);
"""


class LoKitInitializeError(Exception):
    pass


class LoKitImportError(Exception):
    pass


class LoKitExportError(Exception):
    pass


class Document(object):
    def __init__(self, doc, office):
        self.doc = doc
        self.office = office

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        self.doc.pClass.destroy(self.doc)

    def saveAs(self, url, fmt=None, options=None):
        ffi = self.office.ffi
        if fmt:
            fmt = six.b(fmt)
        else:
            fmt = ffi.NULL
        if options:
            options = six.b(options)
        else:
            options = ffi.NULL
        saved = self.doc.pClass.saveAs(self.doc, six.b(url), fmt, options)
        if not saved:
            raise LoKitExportError(self.office.getError())
        return saved


class Office(object):
    def __init__(self, lo_path):
        ffi = FFI()
        ffi.cdef(LOKIT_CDEFS)
        lo = None
        for lib in TARGET_LIB:
            libsoffice = os.path.join(lo_path, lib)
            if os.path.exists(libsoffice):
                lo = ffi.dlopen(libsoffice)
                break

        if not lo:
            raise LoKitInitializeError("Failed to initialize LibreOfficeKit")

        self.ffi = ffi
        self.lokit = lo.libreofficekit_hook(six.b(lo_path))

    def documentLoad(self, url):
        doc = self.lokit.pClass.documentLoad(self.lokit, six.b(url))
        if not doc:
            raise LoKitImportError(self.getError())
        return Document(doc, self)

    def getError(self):
        return self.ffi.string(self.lokit.pClass.getError(self.lokit))

    def __enter__(self):
        return self

    def __exit__(self, type, value, tb):
        if hasattr(self, "lokit"):
            self.lokit.pClass.destroy(self.lokit)



import unittest
import os

from lokit import Office, LoKitInitializeError, LoKitImportError, LoKitExportError

TEST_DIR = os.path.dirname(__file__)

class LokitTest(unittest.TestCase):
    def setUp(self):
        self.lo_path = os.getenv('LO_PATH', '/usr/lib/libreoffice/program/')
        self.test_doc = self._to_abspath("tests/foo.doc")
        self.test_out_rtf = self._to_abspath("tests/out.rtf")
        self.test_out_docx = self._to_abspath("tests/out.docx")
        self.test_out_pdf = self._to_abspath("tests/out.pdf")
        self.test_lockfile = self._to_abspath(".~lock.foo.doc#")

    def _to_abspath(self, path):
        return os.path.abspath(os.path.join(TEST_DIR, path))

    def test_init_no_lo_path(self):
        self.assertRaises(LoKitInitializeError, Office, "wronglopath")

    def test_init(self):
        lo = Office(self.lo_path)
        self.assertIsNotNone(lo)

    def test_no_input_file(self):
        lo = Office(self.lo_path)
        self.assertRaises(LoKitImportError, lo.documentLoad, "foo")

    def test_no_output_file(self):
        lo = Office(self.lo_path)
        with lo.documentLoad(self.test_doc) as doc:
            self.assertRaises(LoKitExportError, doc.saveAs, "")

    def test_wrong_filter(self):
        with Office(self.lo_path) as lo:
            with lo.documentLoad(self.test_doc) as doc:
                self.assertRaises(LoKitExportError, doc.saveAs, self.test_out_rtf, fmt="foobar")

    def test_wrong_options(self):
        with Office(self.lo_path) as lo:
            with lo.documentLoad(self.test_doc) as doc:
                self.assertRaises(LoKitExportError, doc.saveAs, self.test_out_rtf, options="foobar")

    def test_filter_and_options(self):
        with Office(self.lo_path) as lo:
            with lo.documentLoad(self.test_doc) as doc:
                doc.saveAs(self.test_out_docx, fmt="docx", options="SkipImages")
                os.unlink(self.test_out_docx)

    def test_multiple_calls(self):
        with Office(self.lo_path) as lo:
            with lo.documentLoad(self.test_doc) as doc:
                doc.saveAs(self.test_out_docx, fmt="docx", options="SkipImages")
                doc.saveAs(self.test_out_pdf)
                os.unlink(self.test_out_docx)
                os.unlink(self.test_out_pdf)
            with lo.documentLoad(self.test_doc) as doc:
                doc.saveAs(self.test_out_pdf)
                os.unlink(self.test_out_pdf)

    def tearDown(self):
        assert os.path.exists(self.test_lockfile) == False


if __name__ == '__main__':
    unittest.main()
